#!/usr/local/bin/tclsh
# Generate the short flat listing of a single module as plain text.

source "$env(SATHER_HOME)/Browser/Web/common_funcs"

proc run { } {
    global infoFile gAllClasses argv

    set includeComments true
    set noseparation false

    if { [llength ${argv}] < 1 } {
	puts "ERROR:No module specified"
	puts "Usage: shortflat modulename \[-nocomment\] \[-noseparation\]"
	puts " -nocomment: Comments should not be output"
	puts " -noseparation: Avoid whitespace and inter-class lines"
	exit -1
    } else {
	set fullModuleName [lindex ${argv} 0]
	# Check whether the module exists
	set moduleName [getFileName ${fullModuleName}]
	if [file exists ${fullModuleName}] {
	    puts "Generating the short flat form of the classes in module ${moduleName} "
	    set genFile "${moduleName}.shortflat"
	    puts "Generating ${genFile}"
	} else {
	    puts "ERROR:Could not file module named: ${moduleName}"
	    puts "Usage: shortflat modulename \[-nocomment\] \[-noseparation\]"
	    exit -1
	}
	    
    }
    foreach arg ${argv} {
	switch -regexp -- ${arg} {
	    -nocomment {set includeComments false}
	    -noseparation  {set noseparation true}
	}
    }
    # Create gModToClasses
    makeModToClassesTable ${gAllClasses} 
    global gModToClasses
    set names [array names gModToClasses]
    puts "${names}"

    # Genrate short form
    genShortFlatForm ${genFile} ${fullModuleName} ${includeComments} ${noseparation}

}


debugPuts "sourcing getShortFlatForm"
proc genShortFlatForm { toFileName moduleName  includeComments noseparation } {
    # Write the short flat form of all classes in the current 
    # gen_bs_info_raw_dump.tcl to "toFile"
    global gClassDef gModToClasses

    set toFile [open ${toFileName} w]
    set classes  $gModToClasses($moduleName)

    set sortedClasses [lsort -increasing -ascii ${classes}]

    foreach class ${sortedClasses} {
	#  Dump the class and all the routine signatures in it
	set classDef $gClassDef(${class})
	puts "Processing ${class} ..."
	
	# puts ${toFile} "class ${class} is"
	set classFileName [lindex ${classDef} 0]
	set classLineOffset [lindex ${classDef} 1] 
	set classLine [expr ${classLineOffset} - 1]
	set classTxt [getDefinition ${classFileName} ${classLine} ${includeComments} ${noseparation}]
	puts ${toFile} ${classTxt}
	set features [lindex ${classDef} 2] 
	set sortedFeatures [lsort -increasing -ascii ${features}]
	foreach feature ${sortedFeatures} {
	    set featureSig [lindex ${feature} 0]
	    set featureFile [lindex ${feature} 1]
	    set featureLine [lindex ${feature} 2]
	    set featurePermissions [lindex ${feature} 3]
	    set isPriv [string index ${featurePermissions} 1]
	    set isAttrWriter [string index ${featurePermissions} 2]
	    set isAttrReader [string index ${featurePermissions} 3]
	    set isSharedWriter [string index ${featurePermissions} 4]
	    set isSharedReader [string index ${featurePermissions} 5]
	    if { ${isPriv} == "p"} {
		# Do nothing for now
	    } else {
		if { ${isAttrWriter} != "n"} {
		    # Do nothing - reader must be public too
		} elseif { ${isSharedWriter} != "n"} {
		    # Do nothing - reader must be public too
		} else {
		    # Print out routine defintion
		    set featureLine [expr ${featureLine} - 1]
		    set routineDef \
			    [getDefinition ${featureFile} ${featureLine} ${includeComments} ${noseparation} ] 
		    # puts ${toFile} "\t${featureSig}"
		    puts ${toFile} "${routineDef}"
		}
	    }
	    flush ${toFile}
	}
	if { ${noseparation} != "true" } {
	    puts ${toFile} "=============================================================="
	}
    }

    close ${toFile}
}

debugPuts "sourcing getDefinition"
proc getDefinition { fileName routineLoc includeComments noseparation } {
    set fileText [getAllFile $fileName]
    set lines [split $fileText "\n"]
    set firstLine [lindex $lines $routineLoc]
    regsub "private" ${firstLine} "" firstLine
    if { ${noseparation} == "true" } {
	set retLines "${firstLine}"
    } else {
	set retLines "${firstLine} \n"
    }
    
    if { ${includeComments} == "true" } {
	# Do nothing
    } else {
	return ${retLines}
    }
    incr routineLoc
    set nextLine [lindex $lines $routineLoc]
    set nextLineTrim [string trimleft ${nextLine}]
    set isCom [string first "--" $nextLineTrim]
    set isInc [string first "include" $nextLineTrim]
    #    puts "Next line: $nextLine $isCom $isInc"
    while { [expr (($isCom == 0) || ($isInc >= 0))] } {
	incr routineLoc
	if { $isCom == 0 } {
	    set retLines "$retLines$nextLine \n"
	}
	set nextLine [lindex ${lines} ${routineLoc}]
	set nextLineTrim [string trimleft [lindex $lines $routineLoc]]
	set isCom [string first "--" $nextLineTrim]
	set isInc [string first "include" $nextLineTrim]
    }
    return $retLines

}



run

