/* 
 * Copyright (C) International Computer Science Institute, 1995/1996.  COPYRIGHT
 * NOTICE: This code is provided "AS IS" WITHOUT ANY WARRANTY and is subject
 * to the terms of the SATHER LIBRARY GENERAL PUBLIC LICENSE contained in
 * the file "Doc/License" of the Sather distribution.  The license is also
 * available from ICSI, 1947 Center St., Suite 600, Berkeley CA 94704, USA.
 * ---> Please email bugs to "psather-bugs@icsi.berkeley.edu". 
 *
 * Some useful functions for debugging
 *
 * Version 1.0 (released for 1.0.9) by Claudio Fleiner (fleiner@icsi.berkeley.edu)
 */
#include <stdio.h>
#include <sys/time.h>
#include <varargs.h>
#include "pSather.h"
#include "local.h"
#include <am.h>

#ifdef DEBUG
int fprintf(FILE *,const char *,...);
int printf(const char *,...);
int gettimeofday(struct timeval *,void *);
int fflush(FILE *);
thread_t thr_ps_id(void);

/* void debug(const char *file,long line,const char *cc,long a,long b,long c,long d,long e,long f)  */
void debug(va_alist)
va_dcl
{
	va_list ap;
	static lock_t lck=NULL;
	struct timeval now;
	extern int start_sec;
	extern void init_thread();
	char p[30],p2[30];
	char x[200];
	char *file;
	long line;
	char *form;

	va_start(ap);
	file=va_arg(ap,char*);
	line=va_arg(ap,long);
	form=va_arg(ap,char *);
	if(THR_LOCAL==NULL) init_thread();
	thr_ps_id(); /* make sure ID is already there (otherwise we deadlock) */
	if(lck==NULL) lck=LCK_CREATE;
	LCK_LOCK(lck);
	gettimeofday(&now,NULL);

	vsprintf(x,form,ap);
	printf("%03ld.%06ld %02d %s %s %s:%ld - %s\n",now.tv_sec-start_sec,
		now.tv_usec,HERE,thr_print_id(thr_id(),p2),thr_print_id(thr_ps_id(),p),file,line,x);
	fflush(stdout);
	LCK_UNLOCK(lck);
}

#endif

#ifdef REGISTER_THREADS
#define MAX_REGS 1000
static struct REG_STRUCT {
	thread_t id;
	thread_t ps_id;
	LOCAL_MEM m;
	char *file;
	int line;
	void (*func)();
	void *arg;
} regs[MAX_REGS];
static lock_t reg_lock=NULL;

void reg_thread(char *file,int line,void (*func)(),void *arg)
{
	int h,f;
	if(reg_lock==NULL) reg_lock=LCK_CREATE;
	LCK_LOCK(reg_lock);
	f=h=THR_HASH(THR_ID)%MAX_REGS;
	while(!THR_SAME_ID(regs[h].id,THR_NO_THREAD)) {
		h=(h+1)%MAX_REGS;
		if(h==f) {
			fprintf(stderr,"you are registering too many threads, ignoring this one\n");
			LCK_UNLOCK(reg_lock);
			return;
		}
	}
	LCK_UNLOCK(reg_lock);
	regs[h].id=THR_ID;
	regs[h].ps_id=thr_ps_id();
	regs[h].m=THR_LOCAL;
	regs[h].file=file;
	regs[h].line=line;
	regs[h].func=func;
	regs[h].arg=arg;
}

void unreg_thread(char *file,int line)
{
	int h,f;
	f=h=THR_HASH(THR_ID)%MAX_REGS;
	while(!THR_SAME_ID(regs[h].id,THR_ID)) {
		h=(h+1)%MAX_REGS;
		if(h==f) return;
	}
	regs[h].id=THR_NO_THREAD;
}

void PRS()
{
	int h;
	char p[30];
	LCK_LOCK(reg_lock);
	for(h=0;h<MAX_REGS;h++) {
		if(!THR_SAME_ID(regs[h].id,THR_NO_THREAD)) {
			printf("%.3d: %s  %s:%d\n",h,thr_print_id(regs[h].id,p),regs[h].file,regs[h].line);
		}
	}
	LCK_UNLOCK(reg_lock);
	fflush(stdout);
}

void PR(int h)
{
	char p[30],p2[30];
	LCK_LOCK(reg_lock);
	if(!THR_SAME_ID(regs[h].id,THR_NO_THREAD)) {
		printf("id: %s    ps_id: %s\n",thr_print_id(regs[h].id,p),thr_print_id(regs[h].ps_id,p2));
		printf("c-file: %s:%d\n",regs[h].file,regs[h].line);
		printf("sather: %s:%d\n",regs[h].m->file,(int)regs[h].m->line);
#ifdef PRINT_BACKTRACE
		printf("trace:  p PT(0x%p)\n",regs[h].m->pFF);
#endif
		printf("function: p 0x%p(0x%p)\n",regs[h].func,regs[h].arg);
		printf("local: p *(LOCAL_MEM)0x%p\n",regs[h].m);
	} else printf("Thread has continued its mission\n");
	LCK_UNLOCK(reg_lock);
	fflush(stdout);
}

#endif
